/**********************************************************************************************************************
 * @file            CIA.h
 *
 * @brief           The Configuration Interface Application (CIA) registers function handlers with the Lion Protocol
 *                  Processor (LPP) to handle EMS/CCU/DCU/PMU configuration operations.
 *
 * @author          Abram Early
 * @date            08/14/2023
 * @copyright       2023 Lion Energy LLC.
 *                  All rights reserved
 *
 *********************************************************************************************************************/
/**
 * @addtogroup      CIA
 *                  Configuration Interface Application Module
 *
 * @ref             CIA.h
 *
 *********************************************************************************************************************/
#include "COD_Compiler_Defines.h"
#include "lion/icc/ICC.h"
#include "lion/rtos/RTOS_Timeout.h"

#ifndef LION_CIA_H__
#define LION_CIA_H__

#ifdef __cplusplus
extern "C"
{
#endif

    /**********************************************************************************************************************
     * @cond                                              GLOBAL SCOPE                                             @endcond
     *********************************************************************************************************************/

    /**
     * @brief Initialize the CIA module. This should only be called once.
     * @warning No other functions in the module should be called until CIA_Init is called.
     *
     * @param ptr_ICC Pointer to a working ICC context
     *
     */
    GlobScop void CIA_Init(ICC_Context_t *ptr_ICC);

    /**
     * @brief Starts the CIA module
     *
     * @returns 0 on success,
     * @returns -EALREADY if already running.
     */
    GlobScop int CIA_Start();

    /**
     * @brief
     * @param tim_Timeout Timeout to wait for module to shutdown, or RTOS_NO_WAIT to return immediately regardless of success.
     *
     * @returns 0 on success or if tim_Timeout is RTOS_NO_WAIT
     * @returns -EAGAIN if timed out waiting for module to shutdown
     */
    GlobScop int CIA_Stop(RTOS_Timeout_t tim_Timeout);

#ifdef __cplusplus
}
#endif

#endif /* LION_CIA_H__ */