#include "CIA_Internal.h"
#include "lion/LPP.h"
#include "lion/log/log.h"
#include "stdio.h"
#include <fstream>
#include <iostream>
#include <zcbor_decode.h>
#include <zcbor_encode.h>

LOG_MODULE_DECLARE(cia, LOG_LEVEL_DBG);

FileScop int cia_SetSettings(const char *sz_Type, JsonObject ref_Desired, JsonObject ref_Current)
{
    StackScop uint8_t arr_CBORBuffer[4096];
    ZCBOR_STATE_E(str_CBORState, 1, arr_CBORBuffer, sizeof(arr_CBORBuffer), 3);
    zcbor_tstr_encode_ptr(str_CBORState, sz_Type, strlen(sz_Type) + 1);
    StackScop int32_t i32_Index = -1;
    if (ref_Desired.containsKey("id"))
    {
        if (ref_Desired["id"].is<const char *>())
        {
            i32_Index = atoi(ref_Desired["id"]);
        }
        else
        {
            i32_Index = ref_Desired["id"] | (uint32_t)0;
        }
        zcbor_uint32_put(str_CBORState, i32_Index);
    }
    zcbor_map_start_encode(str_CBORState, ref_Desired.size());
    for (ArduinoJson::JsonPair &&str_Element : ref_Desired)
    {
        StackScop const char *sz_Key = str_Element.key().c_str();
        if (strcmp(sz_Key, "id") == 0)
        {
            continue;
        }
        StackScop const char *sz_Value = str_Element.value().as<const char *>();
        zcbor_tstr_encode_ptr(str_CBORState, sz_Key, strlen(sz_Key) + 1);
        zcbor_tstr_encode_ptr(str_CBORState, sz_Value, strlen(sz_Value) + 1);
        // zcbor_nil_put()
    }
    zcbor_map_end_encode(str_CBORState, ref_Desired.size());
    StackScop size_t siz_CBORLength = (str_CBORState[0].payload - arr_CBORBuffer);

    StackScop uint8_t arr_CBORResponse[4096];
    StackScop size_t  siz_CBORResponseLength = sizeof(arr_CBORResponse);
    StackScop int  err_RC = ICC_Function_Call(cia_ptr_ICC, 0x02000201, arr_CBORBuffer, siz_CBORLength, arr_CBORResponse,
                                              &siz_CBORResponseLength, RTOS_SECONDS(3));
    StackScop bool bol_OkayResponse = (err_RC == 0);

    ZCBOR_STATE_D(str_CBORDecode, 1, arr_CBORResponse, siz_CBORResponseLength, 1);
    bol_OkayResponse = bol_OkayResponse && zcbor_list_start_decode(str_CBORDecode);

    for (ArduinoJson::JsonPair &&str_Element : ref_Desired)
    {
        const char *sz_Key   = str_Element.key().c_str();
        const char *sz_Value = str_Element.value().as<const char *>();
        if (strcmp(sz_Key, "id") == 0)
        {
            continue;
        }

        /* This is a bit of tricky slight-of-hand. If `bol_OkayResponse` is false for whatever reason, then
         * `zcbor_int32_decode` won't run leaving s32_Result with it's default value. This allows me to properly handle
         * valid responses, a failed ICC call and invalid response formats with the same code. */
        StackScop int32_t s32_Result = err_RC;
        bol_OkayResponse             = bol_OkayResponse && zcbor_int32_decode(str_CBORDecode, &s32_Result);

        if (s32_Result == 0)
        {
            ref_Current[str_Element.key()] = str_Element.value();
            ref_Desired[str_Element.key()] = "succ";
            LOG_INF("Set \"%s\": \"%s\"", sz_Key, sz_Value);
        }
        else
        {
            ref_Desired[str_Element.key()] = "fail";
            LOG_ERR("Failed to set \"%s\": \"%s\" %d \"%s\"", sz_Key, sz_Value, s32_Result, strerror(abs(s32_Result)));
        }
    }

    bol_OkayResponse = bol_OkayResponse && zcbor_list_end_decode(str_CBORDecode);

    return 0;
}

FileScop void cia_JSONDiff(JsonObject &ref_Target, const JsonObject &ref_Source) {}

/**
 * @brief A helper struct to get the number of bytes processed as JSON.
 */
struct cia_LPP_Reader
{
    RTOS_Timeout_t tim_Timeout;
    LPP_Request_t *ptr_Request;

  public:
    explicit cia_LPP_Reader(LPP_Request_t *ptr_Request, RTOS_Timeout_t tim_Timeout)
        : ptr_Request(ptr_Request), tim_Timeout(RTOS_TIMEOUT_ABS(tim_Timeout))
    {
    }
    int read()
    {
        char chr_Read;
        return readBytes(&chr_Read, 1) == 1 ? chr_Read : 0;
    }
    size_t readBytes(char *buffer, size_t length)
    {
        size_t siz_ReadLength = length;
        (void)LPP_Receive(ptr_Request, buffer, &siz_ReadLength, tim_Timeout);
        return siz_ReadLength;
    }
};

struct cia_LPP_Writer
{
    RTOS_Timeout_t tim_Timeout;
    LPP_Request_t *ptr_Request;

  public:
  public:
    explicit cia_LPP_Writer(LPP_Request_t *ptr_Request, RTOS_Timeout_t tim_Timeout)
        : ptr_Request(ptr_Request), tim_Timeout(RTOS_TIMEOUT_ABS(tim_Timeout))
    {
    }

    size_t write(uint8_t c) { return write(&c, 1); }

    size_t write(const uint8_t *s, size_t n)
    {
        size_t siz_WriteLength = n;
        (void)LPP_ResponseSend(ptr_Request, s, &siz_WriteLength, tim_Timeout);
        return siz_WriteLength;
    }
};

FileScop void cia_SaveSettings(ArduinoJson::JsonDocument &ref_Document)
{
    std::fstream         cls_Stream("settings.json", std::ios::out);
    ArduinoJson::serializeJson(ref_Document, cls_Stream);
}

FileScop void cia_LoadSettings(ArduinoJson::JsonDocument &ref_Document)
{
    std::fstream         cls_Stream("settings.json", std::ios::in);
    DeserializationError cls_Error = ArduinoJson::deserializeJson(ref_Document, cls_Stream);
    if (cls_Error != DeserializationError::Ok)
    {
        ref_Document.to<JsonObject>();
    }
}

FileScop JsonObject cia_FindId(JsonArray ref_Array, JsonVariant ref_Id)
{
    for (JsonObject &&ref_Item : ref_Array)
    {
        if (ref_Item["id"] == ref_Id)
        {
            return ref_Item;
        }
    }
    JsonObject cls_Object = ref_Array.add().to<JsonObject>();
    cls_Object["id"]      = ref_Id;
    return cls_Object;
}

GlobScop int cia_FunctionHandler_cfg_file_set(LPP_Request_t *ptr_Request)
{
    StackScop DynamicJsonDocument  cls_NewDocument(2048);
    StackScop cia_LPP_Reader       cls_Reader(ptr_Request, RTOS_SECONDS(3));
    StackScop DeserializationError cls_Error = ArduinoJson::deserializeJson(cls_NewDocument, cls_Reader);
    StackScop DynamicJsonDocument  cls_CurrentDocument(2048);
    cia_LoadSettings(cls_CurrentDocument);

    for (JsonPair &&str_NewMember : cls_NewDocument.as<JsonObject>())
    {
        if (str_NewMember.value().is<JsonArray>())
        {
            if (!cls_CurrentDocument.containsKey(str_NewMember.key())) {
                cls_CurrentDocument[str_NewMember.key()].to<JsonArray>();
            }
            for (JsonObject &&ref_NewItem : str_NewMember.value().as<JsonArray>())
            {
                cia_SetSettings(str_NewMember.key().c_str(), ref_NewItem,
                                cia_FindId(cls_CurrentDocument[str_NewMember.key()], ref_NewItem["id"]));
            }
        }
        else if (str_NewMember.value().is<JsonObject>())
        {
            if (!cls_CurrentDocument.containsKey(str_NewMember.key())) {
                cls_CurrentDocument[str_NewMember.key()].to<JsonObject>();
            }
            cia_SetSettings(str_NewMember.key().c_str(), str_NewMember.value(), cls_CurrentDocument[str_NewMember.key()]);
        }
    }

    cia_SaveSettings(cls_CurrentDocument);
    
    LPP_Respond(ptr_Request, cls_NewDocument, 0, RTOS_SECONDS(3));

    return 0;
}

GlobScop int cia_FunctionHandler_cfg_file_get(LPP_Request_t *ptr_Request)
{
    StackScop DynamicJsonDocument cls_CurrentDocument(2048);
    cia_LoadSettings(cls_CurrentDocument);

    LPP_Respond(ptr_Request, {}, ArduinoJson::measureJson(cls_CurrentDocument), RTOS_SECONDS(3));
    StackScop cia_LPP_Writer cls_Writer(ptr_Request, RTOS_SECONDS(3));
    serializeJson(cls_CurrentDocument, cls_Writer);
    return 0;
}