/**********************************************************************************************************************
 * @file            ECT.h
 *
 * @brief           The Ethernet Configuration Transport (ECT) application implements a Lion Protocol Processor (LPP)
 *                  transport via a websocket, and broadcasts its availability using Avahi.
 *
 * @author          Abram Early
 * @date            08/08/2023
 * @copyright       2023 Lion Energy LLC.
 *                  All rights reserved
 *
 *********************************************************************************************************************/
/**
 * @addtogroup      ECT
 *                  Ethernet Configuration Transport (ECT) Application Module
 *
 * @ref             ECT.h
 *
 *********************************************************************************************************************/
#include "COD_Compiler_Defines.h"
#include "lion/rtos/RTOS_Timeout.h"

#ifndef LION_ECT_H__
#define LION_ECT_H__

#ifdef __cplusplus
extern "C"
{
#endif

    /**********************************************************************************************************************
     * @cond                                              GLOBAL SCOPE                                             @endcond
     *********************************************************************************************************************/

    /**
     * @brief Initialize the ECT module. This should only be called once.
     * @warning No other functions in the module should be called until ECT_Init is called.
     *
     * @param ptr_ICC Pointer to a working ICC context
     *
     */
    GlobScop void ECT_Init();

    /**
     * @brief Starts the ECT module
     *
     * @returns 0 on success,
     * @returns -EALREADY if already running.
     */
    GlobScop int ECT_Start();

    /**
     * @brief
     * @param tim_Timeout Timeout to wait for module to shutdown, or RTOS_NO_WAIT to return immediately regardless of success.
     *
     * @returns 0 on success or if tim_Timeout is RTOS_NO_WAIT
     * @returns -EAGAIN if timed out waiting for module to shutdown
     */
    GlobScop int ECT_Stop(RTOS_Timeout_t tim_Timeout);

#ifdef __cplusplus
}
#endif

#endif /* LION_ECT_H__ */