/**********************************************************************************************************************
* @file            PMS.h
*
* @brief           This component acts as a modbus server for handling PCS modbus requests from a RTU client device.
*
* @author          Devan Hone
* @date            07/24/2024
* @copyright       2024 Lion Energy LLC.
*                  All rights reserved
*
*********************************************************************************************************************/
/**
* @addtogroup      PMS
*                  Identifier Description for the PCS Modbus Server (PMS) Module  
* 
* @ref             PMS.h 
*                  
*********************************************************************************************************************/
#include "COD_Compiler_Defines.h"

#ifndef PMS_H__
#define PMS_H__

#include <lion/rtos/RTOS_Timeout.h>

#ifdef __cplusplus
extern "C"
{
#endif

/*********************************************************************************************************************
* @cond                                              GLOBAL SCOPE                                             @endcond
*********************************************************************************************************************/

/**
 * @brief Initializes the PMS application.
 * 
 * @returns EXIT_SUCCESS or 0 on success, -EALREADY if the application has already been initialized.
 */
GlobScop int PMS_Init(void);

/**
 * @brief Starts the PMS application thread.
 * 
 * @returns EXIT_SUCCESS or 0 on success, 
 *          -EPERM if the application has not been initialized, 
 *          -EALREADY if the thread is already running. 
 */
GlobScop int PMS_Start(void);

/**
 * @brief Stops the PMS application thread.
 * 
 * @param tim_Timeout The time to wait for the server and thread to stop.
 * 
 * @returns EXIT_SUCCESS or 0 on success, standard posix error code on failure.
 */
GlobScop int PMS_Stop(RTOS_Timeout_t tim_Timeout);

#ifdef __cplusplus
}
#endif

#endif /* PMS_H__ */