#ifndef LION_ICC_ICC_INTERNAL_H__
#define LION_ICC_ICC_INTERNAL_H__

/* ======================================================================= */
/* =========================== I N C L U D E S =========================== */
/* ======================================================================= */

#include "lion/icc/ICC.h"
#include <stddef.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**********************************************************************************************************************
 * @cond                                              MODULE SCOPE                                             @endcond
 *********************************************************************************************************************/

#define icc_u32BIT(bit)              ((uint32_t)(1U << (bit)))
#define icc_u32BIT_MASK(bits, start) ((uint32_t)(((1U << (bits)) - 1U) << (start)))
#define icc_u32XMIT_MASK             icc_u32BIT_MASK(CONFIG_ICC_u32XMIT_BUFFER_COUNT, 0U)
#define icc_u32XMIT_INDEX(index)     ((uint32_t)(0U + (index)))
#define icc_u32XMIT_BIT(index)       icc_u32BIT(icc_u32XMIT_INDEX(index))
#define icc_u32RECV_MASK             icc_u32BIT_MASK(CONFIG_ICC_u32RECV_BUFFER_COUNT, CONFIG_ICC_u32XMIT_BUFFER_COUNT)
#define icc_u32RECV_INDEX(index)     ((uint32_t)(CONFIG_ICC_u32XMIT_BUFFER_COUNT + (index)))
#define icc_u32RECV_BIT(index)       icc_u32BIT(icc_u32RECV_INDEX(index))

    ModScop void          icc_EnqueuePacket(ICC_Context_t *iccContext, ICC_Packet_t *packet);
    ModScop ICC_Packet_t *icc_AllocateTXPacket(ICC_Context_t *iccContext, uint16_t length);
    ModScop ICC_Packet_t *icc_AllocateRXPacket(ICC_Context_t *iccContext, uint16_t length);
    ModScop void          icc_ReleasePacket(ICC_Context_t *iccContext, ICC_Packet_t *packet);

    /**
     * @brief Reserves a packet buffer to be used for reception with the requested length
     * @param iccContext
     * @param length
     * @warning Every instance of icc_ReserveRXPacket MUST be followed by an icc_ReleasePacket
     * @return
     */
    ModScop ICC_Packet_t *icc_ReserveRXPacket(ICC_Context_t *ptrICCContext, uint16_t u16Length);
    ModScop ICC_Packet_t *icc_ReserveTXPacket(ICC_Context_t *ptrICCContext, uint16_t u16Length);
    ModScop void          icc_ReleasePacket(ICC_Context_t *iccContext, ICC_Packet_t *packet);

    ModScop bool icc_Event_ProcessPacket(ICC_Context_t *context, ICC_Packet_t *ptr_Packet);
    ModScop bool icc_Function_ProcessFunctionPacket(ICC_Context_t *ptrContext, ICC_Packet_t *ptrReqPacket);
    ModScop bool icc_Function_ProcessReturnPacket(ICC_Context_t *ptrContext, ICC_Packet_t *ptrRspPacket);

    
ModScop void icc_transport_ForwardTXPacket(ICC_Context_t *icc_Context, ICC_Transport_t *ptr_Transport,
                                          ICC_Packet_t *ptr_Packet);

#ifdef __cplusplus
}
#endif

#endif /* LION_ICC_ICC_INTERNAL_H__ */