#include "lion/TSS.h"
#include "lion/log/log.h"
#include <time.h>
#include <unistd.h>
#include <zcbor_encode.h>

LOG_MODULE_REGISTER(tss, LOG_LEVEL_INF);

FileScop ICC_Function_Handler_t tss_str_TSSHandler;

FileScop int tss_ICCGetCurrentTime(uint32_t u32Id, const void *ptrData, uint32_t u32DataLength, void *ptrResponse,
                                   uint32_t *u32ResponseLength, void *ptrUserData)
{
    StackScop struct timespec str_timespec;
    StackScop int             int_RC = clock_gettime(CLOCK_TAI, &str_timespec);
    if (int_RC != 0)
    {
        int_RC = errno;
        LOG_ERR("Unable to retrieve current time (%s)", strerror(int_RC));
        return -int_RC;
    }
    StackScop struct tm *str_TM = localtime(&str_timespec.tv_sec);

    StackScop      ZCBOR_STATE_E(ptr_Encode, 1, ptrResponse, *u32ResponseLength, 1);
    StackScop bool bol_OK = zcbor_map_start_encode(ptr_Encode, 11);
    bol_OK = bol_OK && zcbor_tstr_put_lit(ptr_Encode, "sec") && zcbor_int32_put(ptr_Encode, str_TM->tm_sec);
    bol_OK = bol_OK && zcbor_tstr_put_lit(ptr_Encode, "min") && zcbor_int32_put(ptr_Encode, str_TM->tm_min);
    bol_OK = bol_OK && zcbor_tstr_put_lit(ptr_Encode, "hour") && zcbor_int32_put(ptr_Encode, str_TM->tm_hour);
    bol_OK = bol_OK && zcbor_tstr_put_lit(ptr_Encode, "mday") && zcbor_int32_put(ptr_Encode, str_TM->tm_mday);
    bol_OK = bol_OK && zcbor_tstr_put_lit(ptr_Encode, "mon") && zcbor_int32_put(ptr_Encode, str_TM->tm_mon);
    bol_OK = bol_OK && zcbor_tstr_put_lit(ptr_Encode, "year") && zcbor_int32_put(ptr_Encode, str_TM->tm_year);
    bol_OK = bol_OK && zcbor_tstr_put_lit(ptr_Encode, "wday") && zcbor_int32_put(ptr_Encode, str_TM->tm_wday);
    bol_OK = bol_OK && zcbor_tstr_put_lit(ptr_Encode, "yday") && zcbor_int32_put(ptr_Encode, str_TM->tm_yday);
    bol_OK = bol_OK && zcbor_tstr_put_lit(ptr_Encode, "isdst") && zcbor_int32_put(ptr_Encode, str_TM->tm_isdst);
# ifdef __USE_MISC
    bol_OK = bol_OK && zcbor_tstr_put_lit(ptr_Encode, "gmtoff") && zcbor_int32_put(ptr_Encode, str_TM->tm_gmtoff);
    bol_OK = bol_OK && zcbor_tstr_put_lit(ptr_Encode, "zone") && zcbor_tstr_put_term(ptr_Encode, str_TM->tm_zone);
# endif
    bol_OK = bol_OK && zcbor_map_end_encode(ptr_Encode, 11);
    if (bol_OK)
    {
        *u32ResponseLength = ptr_Encode->payload - (uint8_t *)ptrResponse;
        return 0;
    }
    else
    {
        *u32ResponseLength = 0;
        return -ENOMEM;
    }
}

GlobScop int TSS_Init(ICC_Context_t *ptr_ICC)
{
    ICC_Function_InitHandler(&tss_str_TSSHandler, 0x01000001, 0xFFFFFFFF, &tss_ICCGetCurrentTime, NULL);
    ICC_Function_RegisterHandler(ptr_ICC, &tss_str_TSSHandler);
}