#include <stdio.h>
#include <unistd.h>
#include <termios.h>
#include <systemd/sd-daemon.h>

#include "lion/ICC_Client.h"
#include "lion/ial/IAL.h"
#include "lion/log/log.h"
#include "lion/PMS.h"
#include "lion/LPP.h"
#include "lion/ECT.h"
#include "lion/CIA.h"
#include "lion/TSS.h"
#include "app_version.h"

#define STRINGIFY(X) __STRING(X)

LOG_MODULE_REGISTER(main, LOG_LEVEL_INF);

ICC_Context_t *ptr_IccCtx;

int main(int argc, char *argv[])
{
    int err=0;
    
    LOG_INF("Program Started v" APP_VERSION_STRING " [" STRINGIFY(APP_BUILD_VERSION) "]\n");

    ptr_IccCtx = ICC_Client_Init(NULL);
    ICC_Client_Start(RTOS_SECONDS(3));

    IAL_Init(ptr_IccCtx);
    LPP_Init();
    ECT_Init();
    PMS_Init();
    TSS_Init(ptr_IccCtx);
    LPP_Start();
    ECT_Start();

    CIA_Init(ptr_IccCtx);

    PMS_Start();

    while (1)
    {
        sleep(1);
        sd_notify(0, "WATCHDOG=1"); /* Feed the Hungry Watchdog */
        fflush(stdout);
    }

    return 0;
}